/*
 * GkSu -- Gtk+ Frontend to su and sudo
 * Copyright (C) 2002 Gustavo Noronha Silva
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <X11/Xlib.h>

#include "defines.h"

#include "gksu.h"
#include "xauth.h"
#include "pixmaps.h"
#include "gui.h"

void
ask_pass_ok_cb (GtkWidget *w, gpointer data)
{
  GtkWidget *dialog = (GtkWidget*)data;

  gtk_dialog_response (GTK_DIALOG(dialog),
		       GTK_RESPONSE_OK);
}

gchar*
ask_password (GkConf *conf, gchar *format, ...)
{
  gchar *pass = NULL;
	
  GtkWidget
    *dialog, 
    *hbox,
    *image,
    *entry, 
    *label;

  gint response;

  va_list ap;
  gchar *msg;

  va_start(ap, format);
  msg = g_strdup_vprintf(format, ap);
  va_end(ap);

  /* dialog window */
  dialog = gtk_dialog_new_with_buttons (conf->title, NULL, GTK_DIALOG_MODAL,
					GTK_STOCK_CANCEL,
					GTK_RESPONSE_CANCEL,
					GTK_STOCK_OK,
					GTK_RESPONSE_OK,
					NULL);

  gtk_dialog_set_has_separator (GTK_DIALOG(dialog), FALSE);
  /* Center window and prepare for grab */
  gtk_window_set_default_size (GTK_WINDOW(dialog), 250, 100);
  gtk_window_set_position (GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
  gtk_window_stick (GTK_WINDOW(dialog));

  /* hbox */
  hbox = gtk_hbox_new (FALSE, 2);
  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox),
		     hbox, TRUE, TRUE, 2);
  gtk_widget_show (hbox);

  /* image */
  image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_QUESTION,
				    GTK_ICON_SIZE_DIALOG);
  gtk_box_pack_start (GTK_BOX(hbox), image, TRUE, TRUE, 0);
  gtk_widget_show (image);

  /* label */
  label = gtk_label_new (msg);
  g_free (msg);
  gtk_box_pack_start (GTK_BOX(hbox), label, TRUE, TRUE, 0);
  gtk_widget_show (label);

  /* entry */
  entry = gtk_entry_new();
  gtk_signal_connect (GTK_OBJECT(entry), "activate",
		      GTK_SIGNAL_FUNC(ask_pass_ok_cb),
		      dialog);
  gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox),
		      entry, TRUE, TRUE, 2);
  gtk_widget_show (entry);
  
  gtk_widget_show_all (dialog);
  gtk_widget_grab_focus(entry);

  while (gtk_events_pending())
    gtk_main_iteration();

  if (conf->grab)
    {
      /* Grab focus */
      XGrabServer(GDK_DISPLAY());
      if (gdk_pointer_grab(dialog->window, TRUE, 0, NULL, NULL, 
			   GDK_CURRENT_TIME))
	{
	  gdk_pointer_ungrab(GDK_CURRENT_TIME);
	  XUngrabServer(GDK_DISPLAY());
	  gtk_widget_hide (dialog);
	  report_failed_grab ();
	  return NULL;
	}
      if (gdk_keyboard_grab(dialog->window, FALSE, GDK_CURRENT_TIME))
	{
	  gdk_pointer_ungrab(GDK_CURRENT_TIME);
	  XUngrabServer(GDK_DISPLAY());
	  gtk_widget_hide (dialog);
	  report_failed_grab ();
	  return NULL;
	}
    }

  /* let the magic begin */
  response = gtk_dialog_run (GTK_DIALOG(dialog));
  
  if (conf->grab)
    {
      /* Ungrab */
      XUngrabServer(GDK_DISPLAY());
      gdk_pointer_ungrab(GDK_CURRENT_TIME);
      gdk_keyboard_ungrab(GDK_CURRENT_TIME);
      gdk_flush();
    }

  /* Report passphrase if user selected OK */
  pass = g_strdup (gtk_entry_get_text(GTK_ENTRY(entry)));
		
  /* Kill the entry's copy of the passphrase. */
  gtk_entry_set_text(GTK_ENTRY(entry), "");
			
  switch (response)
    {
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_DELETE_EVENT:
    case GTK_RESPONSE_NONE:
      free_pass (pass);
      clean_dir (conf->dir);
      exit (0);
      break;
    }

  gtk_widget_hide (dialog);

  while (gtk_events_pending())
    gtk_main_iteration();

  return pass;
}

void
report_failed_grab (void)
{
  gk_dialog (GTK_MESSAGE_ERROR, 
	     _("Could not grab keyboard or mouse.\n"
	       "A malicious client may be eavesdropping\n"
	       "on your session."));
}

/* 
   Shows 'msg' in a dialog box with an OK button 
   This function is to be a helper for functions needing to
   display some information to the user
*/
void 
gk_dialog (GtkMessageType type, gchar *format, ...)
{
  GtkWidget *diag_win;

  va_list ap;
  gchar *msg;

  va_start(ap, format);
  msg = g_strdup_vprintf(format, ap);
  va_end(ap);

  diag_win = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
				     type, GTK_BUTTONS_CLOSE,
				     msg);
			     
  gtk_signal_connect_object (GTK_OBJECT(diag_win), "delete_event",
			     GTK_SIGNAL_FUNC(gtk_main_quit), 
			     NULL);
  gtk_window_set_position (GTK_WINDOW(diag_win), GTK_WIN_POS_CENTER);
  gtk_window_set_resizable (GTK_WINDOW(diag_win), FALSE);

  gtk_widget_show_all (diag_win);
  gtk_dialog_run (GTK_DIALOG(diag_win));

  g_free (msg);

  gtk_widget_destroy (diag_win);
}
